# NOTE: This is an auto-generated file. Please do not edit this file manually.
from pathlib import Path

from airflow import DAG
from airflow.utils import timezone
from astro import sql as aql
from astro.table import Table
from sql_cli.dag_render import render_tasks

DAGS_FOLDER = Path(__file__).parent.as_posix()

with DAG(
    dag_id="{{ dag.dag_id }}",
    start_date=timezone.parse("{{ dag.start_date }}"),
    schedule_interval=None,
) as dag:
    render_tasks(Path(f"{DAGS_FOLDER}/include/{{ dag.dag_id }}"))
