CONFIG_DIR = "config"
GLOBAL_CONFIG_DIRECTORY = "global"
CONFIG_FILENAME = "configuration.yml"

DEFAULT_ENVIRONMENT = "default"
DEFAULT_AIRFLOW_HOME = f".airflow/{DEFAULT_ENVIRONMENT}"
DEFAULT_DAGS_FOLDER = ".airflow/dags"

GENERATED_WORKFLOW_INCLUDE_DIRECTORY = "include"
